@@ -1,6 +1,7 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
 /*
- * anjuta-status.c Copyright (C) 2004 Naba Kumar  <naba@gnome.org>
+ * anjuta-status.c
+ * Copyright (C) 2004 Naba Kumar  <naba@gnome.org>
  * 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free 
@@ -14,7 +15,16 @@
  * 
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc., 59 
- * Temple Place, Suite 330, Boston, MA  02111-1307  USA 
+ * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
+ */
+
+/**
+ * SECTION:anjuta-status
+ * @short_description: Program status such as status message, progress etc. 
+ * @see_also: 
+ * @stability: Unstable
+ * @include: libanjuta/anjuta-status.h
+ * 
  */
 
 #include <config.h>
@@ -33,9 +43,9 @@ struct _AnjutaStatusPriv
 	/* Progress bar */
 	gint total_ticks;
 	gint current_ticks;
-	GtkWidget *splash;
 	
 	/* Splash */
+	GtkWidget *splash;
 	gboolean disable_splash;
 	gchar *splash_file;
 	gint splash_progress_position;
@@ -57,6 +67,7 @@ static void on_widget_destroy (AnjutaStatus *status, GObject *widget);
 static void
 anjuta_status_finalize (GObject *widget)
 {
+	g_free(ANJUTA_STATUS(widget)->priv);
 	GNOME_CALL_PARENT(G_OBJECT_CLASS, finalize, (widget));
 }
 
@@ -94,13 +105,18 @@ anjuta_status_dispose (GObject *widget)
 		g_hash_table_destroy (status->priv->widgets);
 		status->priv->widgets = NULL;
 	}
-	
+	if (status->priv->window)
+	{
+		g_object_remove_weak_pointer (G_OBJECT (status->priv->window),
+						   (gpointer*)(gpointer)&status->priv->window);
+		status->priv->window = NULL;
+	}
 	GNOME_CALL_PARENT(G_OBJECT_CLASS, dispose, (widget));
 }
 
 static void
 anjuta_status_instance_init (AnjutaStatus *status)
-{
+{	
 	status->priv = g_new0 (AnjutaStatusPriv, 1);
 	status->priv->splash_file = NULL;
 	status->priv->splash_progress_position = 0;
@@ -307,8 +323,8 @@ anjuta_status_add_widget (AnjutaStatus *status, GtkWidget *widget)
 }
 
 void
-anjuta_status_progress_set_splash (AnjutaStatus *status, const gchar *splash_file,
-								   gint splash_progress_position)
+anjuta_status_set_splash (AnjutaStatus *status, const gchar *splash_file,
+						  gint splash_progress_position)
 {
 	g_return_if_fail (ANJUTA_IS_STATUS (status));
 	g_return_if_fail (splash_file != NULL);
@@ -320,12 +336,18 @@ anjuta_status_progress_set_splash (AnjutaStatus *status, const gchar *splash_fil
 }
 
 void
-anjuta_status_progress_disable_splash (AnjutaStatus *status,
-									   gboolean disable_splash)
+anjuta_status_disable_splash (AnjutaStatus *status,
+							  gboolean disable_splash)
 {
 	g_return_if_fail (ANJUTA_IS_STATUS (status));
 
 	status->priv->disable_splash = disable_splash;
+	if (status->priv->splash)
+	{
+		gtk_widget_destroy (status->priv->splash);
+		status->priv->splash = NULL;
+		anjuta_status_progress_add_ticks (status, 0);
+	}
 }
 
 void
@@ -334,7 +356,7 @@ anjuta_status_progress_add_ticks (AnjutaStatus *status, gint ticks)
 	gfloat percentage;
 	
 	g_return_if_fail (ANJUTA_IS_STATUS (status));
-	g_return_if_fail (ticks > 0);
+	g_return_if_fail (ticks >= 0);
 	
 	status->priv->total_ticks += ticks;
 	if (!GTK_WIDGET_REALIZED (status))
@@ -349,7 +371,6 @@ anjuta_status_progress_add_ticks (AnjutaStatus *status, gint ticks)
 		}
 	}
 	percentage = ((gfloat)status->priv->current_ticks)/status->priv->total_ticks;
-	
 	if (status->priv->splash)
 	{
 		e_splash_set (E_SPLASH(status->priv->splash), NULL, NULL, NULL,
@@ -376,6 +397,30 @@ anjuta_status_progress_add_ticks (AnjutaStatus *status, gint ticks)
 	}
 }
 
+void
+anjuta_status_progress_pulse (AnjutaStatus *status, const gchar *text)
+{
+	GtkProgressBar *progressbar;
+	GtkWidget *statusbar;
+	
+	progressbar = gnome_appbar_get_progress (GNOME_APPBAR (status));
+	statusbar = gnome_appbar_get_status (GNOME_APPBAR (status));
+	
+	if (text)
+		anjuta_status_set (status, "%s", text);
+	
+	gtk_progress_bar_pulse (progressbar);
+	
+	gtk_widget_queue_draw (GTK_WIDGET (statusbar));
+	gtk_widget_queue_draw (GTK_WIDGET (progressbar));
+	if (GTK_WIDGET(progressbar)->window != NULL &&
+		GDK_IS_WINDOW(GTK_WIDGET(progressbar)->window))
+		gdk_window_process_updates (GTK_WIDGET(progressbar)->window, TRUE);
+	if (GTK_WIDGET(statusbar)->window != NULL &&
+		GDK_IS_WINDOW(GTK_WIDGET(statusbar)->window))
+		gdk_window_process_updates (GTK_WIDGET(statusbar)->window, TRUE);
+}
+
 void
 anjuta_status_progress_tick (AnjutaStatus *status,
 							 GdkPixbuf *icon, const gchar *text)
@@ -456,7 +501,7 @@ anjuta_status_set_title_window (AnjutaStatus *status, GtkWidget *window)
 	g_return_if_fail (GTK_IS_WINDOW (window));
 	status->priv->window = GTK_WINDOW (window);
 	g_object_add_weak_pointer (G_OBJECT (window),
-							   (gpointer*)&status->priv->window);
+							   (gpointer*)(gpointer)&status->priv->window);
 }
 
 void
@@ -467,18 +512,16 @@ anjuta_status_set_title (AnjutaStatus *status, const gchar *title)
 	if (!status->priv->window)
 		return;
 	
+	const gchar *app_name = g_get_application_name();
 	if (title)
 	{
-		gchar *str;
-		str = g_strconcat (title, " - ",
-						   g_get_application_name(), NULL);
+		gchar* str = g_strconcat (app_name, " - ", title, NULL);
 		gtk_window_set_title (status->priv->window, str);
 		g_free (str);
 	}
 	else
 	{
-		gtk_window_set_title (status->priv->window,
-							  g_get_application_name ());
+		gtk_window_set_title (status->priv->window, app_name);
 	}
 }
 
